/*------------------------------------------------*/
/* FFTEST : A test program for FFT module         */

#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "FHT.h"		/* Defs for using Fixed-point FFT module */
#include "mydefs.h"
#include "avr/eeprom.h"

/*------------------------------------------------*/
/* Global variables                               */

int16_t capture[FHT_N];
volatile uint16_t pwm_setting[8];
uint8_t pwm_pause; 
uint8_t counter[8];

volatile uint8_t peak[8];

volatile uint8_t no_audio;       //    
volatile uint8_t pause;			//     

volatile static uint8_t faderate = 8; //   
volatile static uint16_t fade = 0;
volatile static uint16_t fadespeed = 640; //      = fadeinitial * faderate
volatile static uint8_t fadeinitial = 80; //   
volatile static uint8_t sensitivity = 1; //   /  
volatile static uint8_t runlight = 1; //     ().   .
volatile static uint8_t ChangeLed = 1; //       ,  "1"     
//volatile static uint8_t ChangeLedLong = 1; //  "Agressive/Non-agressive"   
volatile static uint8_t writecheck = 0; //      EEPROM 

//==================================================+
//     EEPROM					|
//==================================================+
uint8_t EEMEM ee_faderate; //   
uint8_t EEMEM ee_sensitivity; //   /  
uint8_t EEMEM ee_runlight; //     ().   .
uint8_t EEMEM ee_writecheck; //      EEPROM 

const uint16_t pwmtable_10[64]  PROGMEM = { 0, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 4, 4, 5, //     ()
											5, 6, 6, 7, 8, 9, 10, 11, 12, 13, 15, 17,
											19, 21, 23, 26, 29, 32, 36, 40, 44, 49, 55,
											61, 68, 76, 85, 94, 105, 117, 131, 146, 162,
											181, 202, 225, 250, 279, 311, 346, 386, 430,
											479, 534, 595, 663, 739, 824, 918, 1023 };

/*------------------------------------------------*/
/* Timer 1 Output COMPARE A Interrupt             */

ISR( TIMER1_COMPA_vect ) {
		static uint16_t pwm_cnt = 0;
		uint8_t tmp = 0;

		OCR1A += (uint16_t) T_PWM;
		
		//==================================================+
		//   pwm_setting     	|
		// pwm_cnt,     ()	|
		//   , ..   	|
		//   1  0. (  )		|
		//==================================================+
		if(pwm_setting[0] > pwm_cnt) tmp |= LED1;
		if(pwm_setting[1] > pwm_cnt) tmp |= LED2;
		if(pwm_setting[2] > pwm_cnt) tmp |= LED3;
		if(pwm_setting[3] > pwm_cnt) tmp |= LED4;
		if(pwm_setting[4] > pwm_cnt) tmp |= LED5;
		if(pwm_setting[5] > pwm_cnt) tmp |= LED6;
		if(pwm_setting[6] > pwm_cnt) tmp |= LED7;
		if(pwm_setting[7] > pwm_cnt) tmp |= LED8;
	
		if ((runlight != 0) && (pause != 0)) { //    
			if(pwm_pause > pwm_cnt) PORTB |= (1<<PB1);
			else PORTB &= ~(1<<PB1);
		}
		
		LED_PORT = tmp;
	
		//      ( pwm_setting[y] > pwm_cnt )
		if(pwm_cnt == (PWM_STEPS - 1))
		pwm_cnt = 0;
		else
		pwm_cnt++;
		
		//==================================================+
		// ""   peak[]				|
		//==================================================+
		if( fade == fadespeed) { //  ,   .   ,     ""    ""
			if(counter[0] == 0)
			if(peak[0]) peak[0]--;

			if(counter[1] == 0)
			if(peak[1]) peak[1]--;

			if(counter[2] == 0)
			if(peak[2]) peak[2]--;

			if(counter[3] == 0)
			if(peak[3]) peak[3]--;

			if(counter[4] == 0)
			if(peak[4]) peak[4]--;

			if(counter[5] == 0)
			if(peak[5]) peak[5]--;
														
			if(counter[6] == 0)
			if(peak[6]) peak[6]--;

			if(counter[7] == 0)
			if(peak[7]) peak[7]--;
														
			fade = 0;
		}
		fade++;
}

ISR( TIMER1_COMPB_vect ) {
	
	//==================================================+
	//  								|
	//==================================================+
	static unsigned char ScanPeriodButton=5; //   ""  ,    
	/*	
	!!!         .     ( ) 
	        ""  
	  ""  _ms_delay(),          
	  . ..        /      uiScanPeriodButton
	*/	
	static unsigned char almostsec = 0; //"" -   . 1 almostsec = 1300 uiScanPeriodButton, ..    uiScanPeriodButton
	static unsigned char second = 0; //  "" 1  ~ 25 almostsec =)  
	//==================================================+		
	//  , 						|
	//==================================================+
	static unsigned char blink; //  
	static unsigned char longpress; //   
	static unsigned char blinkpause; 
	static unsigned char ChangeLedLong = 0; //       
	static unsigned char bOldButton,bButton; //      

	//==================================================+
	//   							|
	//==================================================+	
	static unsigned char event = 0; //  /
	static unsigned char vector = 0; //  
	static signed char chanelfx; //  
	static unsigned char pwmcountpause = 0; //  
	
	OCR1B += (uint16_t) UINT16_MAX; //  
		
	//==================================================+
	//   "Fade Speed"  "Runlight" 		|
	//==================================================+	
	if (ScanPeriodButton) ScanPeriodButton--;	//    
	else { //       
		
		ScanPeriodButton=5;	//     
				
			if (((PINB & (1<<PB0))) && ((PINC & (1<<PC0))))  bButton = 0; //   
		
			if ((!(PINB & (1<<PB0))) && ((PINC & (1<<PC0))) && ( longpress < 75 )) { //    "Fade Speed" - 1
				bOldButton = 1;
				bButton = 1;
			}
			
			if ((!(PINC & (1<<PC0))) && ((PINB & (1<<PB0))) && ( longpress < 75 )) { //    "Runlight" - 2
				bOldButton = 2;
				bButton = 1;
			}
			
			if ((!(PINB & (1<<PB0))) && (!(PINC & (1<<PC0))) && ( longpress < 75 )) { //    
				bOldButton = 3;
				bButton = 1;
			}
			
			//==================================================+
			//  							 		|
			//==================================================+
			if ((bOldButton != 0) && (bButton == 0) && ( longpress == 0 )) { //           +   
								
				//   "Fade Speed"
				if (bOldButton == 1) { 
						fade = 0;
						faderate--;
						faderate--;
						if (faderate <= 1) faderate = 8;
						if (pause == 0) fadespeed = fadeinitial * faderate;
						ChangeLed = 1; //      		
						bOldButton = 0; //    
				}
				
				//   "Runlight"
				if (bOldButton == 2) { 
											
					runlight++; //  /      
					
					if (runlight == 3) runlight = 0;	
					
					if (runlight == 0) { //   -  ,  ,  
						pause = 0; //      
						second = 0; //    /
						PORTB &= ~(1<<PB1); //   						
					}
					
					if (runlight == 2) { //
						pause = 0; //      
						second = 0; //    /
					}
					
				bOldButton = 0; //    
				}
				
			}
			
		//==================================================+
		//  	( )		 		|
		//==================================================+	
		
		//       
		if (bButton == 1) {
			if (longpress < 75) longpress++; 
		} 
		else {
			longpress = 0; //  ,    
		}
		
		if ((longpress == 75) && (bButton == 1)) { // 75  uiScanPeriodButton  = 3 
			
			if (bOldButton == 1) { //  "Fade Speed"
				
				sensitivity++;
				if (sensitivity == 4) sensitivity = 1; //     
				
				blink = 8; //    
				ChangeLedLong = 1; //    
				bOldButton = 0; // "" 
				PORTB &= ~(1<<PB3);		//    
				PORTB &= ~(1<<PB4);		
				PORTB &= ~(1<<PB5);		
			}
			
			if (bOldButton == 2) { //  "Runlight"

				//   EEPROM  
				eeprom_update_byte(&ee_faderate, faderate);
				eeprom_update_byte(&ee_sensitivity, sensitivity);
				eeprom_update_byte(&ee_runlight, runlight);
							
				blink = 8; //    
				ChangeLedLong = 1; //    
				bOldButton = 0; // "" 
				PORTB &= ~(1<<PB3);		//    
				PORTB &= ~(1<<PB4);
				PORTB &= ~(1<<PB5);
			}
			
			if (bOldButton == 3) { //   
				blink = 30; //    
				ChangeLedLong = 1; //    
				bOldButton = 0; // "" 
				PORTB &= ~(1<<PB3);		//    
				PORTB &= ~(1<<PB4);
				PORTB &= ~(1<<PB5);
				writecheck = 0;
			}
		
		}
		
		//==================================================+
		//   						|
		//==================================================+
		if (ChangeLedLong == 1) { //       
			if (blinkpause == 0) {
				if (!(PORTB & (1<<PB3))) {
					PORTB |= (1<<PB3);		// 
					PORTB |= (1<<PB4);		// 
					PORTB |= (1<<PB5);		// 
				} 
				else {
					PORTB &= ~(1<<PB3);		// 
					PORTB &= ~(1<<PB4);		// 
					PORTB &= ~(1<<PB5);		// 	
				}
				blinkpause = 1;
				blink--; //  
				if (blink == 0) { //       "Fade Speed"
					ChangeLed = 1;
					ChangeLedLong = 0;
				}
				
			} else {
				blinkpause--;
			}
			
		}
		
		//==================================================+
		//   							|
		//==================================================+
		if (bButton == 1) PORTB |= (1<<PB2); //      
		else PORTB &= ~(1<<PB2); //    -  
		
			//==================================================+
			//     					|
			//==================================================+
			if (runlight == 1) { //           ,   
				if ((no_audio == 1) && (pause == 0)) { //           ""
					almostsec++; //  "" =)
					if (almostsec == 40) { //  25  = 1 
						second++; //  
						almostsec = 0;
					}
					if (second == 4) { //  5       
						pause = 1;
						second = 0;
						almostsec = 0;
						fadespeed = 1300;

						pwmcountpause = 0;
						
						if (vector == 0) {
							vector = 1;
							chanelfx = 0;
						}
						else {
							vector = 0;
							chanelfx = 7;
						}
						
					}
				}
				else {//   ,         .   5     1   , ..     5 =)
					if ((no_audio == 0) && (pause == 0)) {
						second = 0;
						pwm_pause = 0;
						PORTB &= ~(1<<PB1);
					}
				}
				
				if (pause == 1) { //    -  ,   
					almostsec++; //  "" =)
					
					if (almostsec == 8) { //   25  = 1 

						event = 1;
						almostsec = 0;

					}
					
					//==================================================+
					//    					|
					//==================================================+
					pwm_pause = pwmcountpause;
					if (pwmcountpause < 100) pwmcountpause++;
					
					//==================================================+
					//   								|
					//==================================================+

					if (event == 1) {

						if (vector == 1) {
							peak[chanelfx] = 63;
							chanelfx++;
							if (chanelfx > 7) chanelfx = 0;
						}
						else {
							peak[chanelfx] = 63;
							chanelfx--;
							if (chanelfx < 0) chanelfx = 7;
						}
						
						event = 0;
					}
					
				}
			}

			
			//==================================================+
			//      		|
			//==================================================+
			if (runlight == 2) { 
				if ((no_audio == 1) && (pause == 0)) { //           ""
					almostsec++; //  "" =)
					if (almostsec == 40) { //  25  = 1 
						second++; //  
						almostsec = 0;
					}
					if (second == 3) { //  4       
						pause = 2;
						second = 0;
						almostsec = 0;	
						pwmcountpause = 0;								
					}
				}
				else {//   ,         .   5     1   , ..     5 =)
					if ((no_audio == 0) && (pause == 0)) {
						second = 0;
						pwm_pause = 0;
						PORTB &= ~(1<<PB1);
					}
				}
				
				//==================================================+
				//   						|
				//==================================================+	
				if (pause == 2)	{
				 	for (uint8_t y = 0; y < 8; y++) {
						pwm_setting[y] = pwmcountpause;	
 					}	 
					 
					//==================================================+
					//    					|
					//==================================================+
					pwm_pause = pwmcountpause;
					if (pwmcountpause < 100) pwmcountpause++; //     
									
				}
				
			}
	}
		//==================================================+
		//  "Agressive Mode"						|
		//==================================================+
		//   "Agressive Mode",     .    -  .    ChangeLed.
		//if (ChangeLedLong == 1) { //          
		if (sensitivity == 1) {   	// if mode is Normal (non-agressive)
			PORTC |= (1<<PC1);		// 
			PORTC &= ~(1<<PC2);		// 
			PORTC &= ~(1<<PC3);		// 
		}
		
		if (sensitivity == 2) {		// if mode is Agressive-1
			PORTC &= ~(1<<PC1);		// 
			PORTC |= (1<<PC2);		// 
			PORTC &= ~(1<<PC3);		// 
		}
		
		if (sensitivity == 3) {		// if mode is Agressive-2
			PORTC &= ~(1<<PC1);		// 
			PORTC &= ~(1<<PC2);		// 
			PORTC |= (1<<PC3);	    // 
		}
		//	ChangeLedLong = 0;
		//}		
	//==================================================+
	//  "Fade Speed"							|
	//==================================================+
	//    ,     .    -  .    ChangeLed.
	if (ChangeLed == 1) { //          
		if (faderate == 8) {   	// if fade step is Nr.1 
			PORTB |= (1<<PB3);		// 
			PORTB |= (1<<PB4);		// 
			PORTB |= (1<<PB5);		// 
		}
		
		if (faderate == 6) {		// if fade step is Nr.2 
			PORTB |= (1<<PB3);		// 
			PORTB &= ~(1<<PB4);		// 
			PORTB &= ~(1<<PB5);		// 
		}
		
		if (faderate == 4) {		// if fade step is Nr.3 
			PORTB &= ~(1<<PB3);		// 
			PORTB |= (1<<PB4);		// 
			PORTB &= ~(1<<PB5);		// 
		}
		
		if (faderate == 2) {      // if fade step is Nr.4 
			PORTB &= ~(1<<PB3);		// 
			PORTB &= ~(1<<PB4);		// 
			PORTB |= (1<<PB5);	    // 
		}
		ChangeLed = 0; 
	}

}

/*------------------------------------------------*/
/* Capture waveform                               */

void capture_wave(int16_t *buffer, uint16_t count) {
	ADMUX = _BV(REFS0)|_BV(ADLAR)|_BV(MUX2)|_BV(MUX0);				// channel

	do {
		#if defined (__AVR_ATmega8__)															
		ADCSRA = _BV(ADEN)|_BV(ADSC)|_BV(ADFR)|_BV(ADIF)|_BV(ADPS2)|_BV(ADPS1);					// = 250 (16000000 /  64) ADPS=110 -  
		#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega644__)							
		ADCSRA = _BV(ADEN)|_BV(ADSC)|_BV(ADATE)|_BV(ADIF)|_BV(ADPS2)|_BV(ADPS1);
		#elif defined (__AVR_ATmega328P__)
		//ADCSRA = _BV(ADEN)|_BV(ADSC)|_BV(ADATE)|_BV(ADIF)|_BV(ADPS2)|_BV(ADPS1);
		ADCSRA = _BV(ADEN)|_BV(ADSC)|_BV(ADATE)|_BV(ADIF)|_BV(ADPS2)|_BV(ADPS1) |_BV(ADPS0);	// = 156 (20000000 / 128) ADPS=111
		#endif
		while(bit_is_clear(ADCSRA, ADIF));
		*buffer++ = ADC - 32768;

	} while(--count);

	ADCSRA = 0;
}

void setdefault (void) {
	eeprom_write_byte(&ee_faderate, 8);
	eeprom_write_byte(&ee_sensitivity, 1);
	eeprom_write_byte(&ee_runlight, 1);
	eeprom_write_byte(&ee_writecheck, 11);
	writecheck = eeprom_read_byte(&ee_writecheck);
	faderate = eeprom_read_byte(&ee_faderate);
	sensitivity = eeprom_read_byte(&ee_sensitivity);
	runlight = eeprom_read_byte(&ee_runlight);
}

/*------------------------------------------------*/
/*                                                */

int main (void) {
	uint16_t s;
	uint16_t maxover = 0;
	uint16_t over = 0;

	LED_DIR  |=  (LED1 | LED2 | LED3 | LED4 | LED5 | LED6 | LED7 | LED8 );
	LED_PORT &= ~(LED1 | LED2 | LED3 | LED4 | LED5 | LED6 | LED7 | LED8 );

	TCCR1B = _BV(CS10);	//      
	TIMSK |= _BV(OCIE1A) | _BV(OCIE1B);	//  
	
	
	PORTB |= (1<<PB0); 		//  "Fade Speed"
	PORTC |= (1<<PC0);		//  "on/off runlight"
	DDRB |= (1<<PB1);		//  " "
	DDRB |= (1<<PB2);		//  -      
	
	DDRB = 0b00111110; // define PB1...PB5 as digital output
	DDRC = 0b00001110; // define PC1...PC3 as digital output (Agressive modes status LEDs)
	
	//==================================================+
	//    EEPROM  			|
	//==================================================+
	writecheck = eeprom_read_byte(&ee_writecheck); //      EEPROM    
	if (writecheck == 11) { //      
		faderate = eeprom_read_byte(&ee_faderate);
		sensitivity = eeprom_read_byte(&ee_sensitivity);
		runlight = eeprom_read_byte(&ee_runlight);
	} 
	else { //            
		setdefault ();
	}

	ChangeLed = 1;
	fadespeed = fadeinitial * faderate;
	
	sei();
	
	for(;;) {
		
		if (writecheck == 0) setdefault();
		//     
		memset( fht_input, 0, sizeof(fht_input) );

		capture_wave(fht_input, FHT_N); // put real data into bins
		fht_window(); // window the data for better frequency response
		fht_reorder(); // reorder the data before doing the fht
		fht_run(); // process the data in the fht
		fht_mag_lin(); // take the output of the fht

		//     
		memset( capture, 0, sizeof(capture) );
				
// 0	0
// 1	1
// 2	4
// 5	10
// 11	18
// 19	35
// 35	49
// 50	63					
				
		//==================================================+
		//    						|
		//==================================================+	
		for (uint16_t n = 0; n < FHT_N / 2; n++) {

			s = fht_lin_out[n];
			
			if((n >= 1)  && (n <= 2)) {
				if((capture[0] + s) <= SCORE_MAX) capture[0] += s;
				else capture[0] = SCORE_MAX;
			} 
			
			if((n >= 3)  && (n <= 5)) {
				if((capture[1] + s) <= SCORE_MAX) capture[1] += s;
				else capture[1] = SCORE_MAX;
			}

			if((n >= 6)  && (n <= 10)) {
				if((capture[2] + s) <= SCORE_MAX) capture[2] += s;
				else capture[2] = SCORE_MAX;
			}
			
			if((n >= 11) && (n <= 19)) {
				if((capture[3] + s) <= SCORE_MAX) capture[3] += s;
				else capture[3] = SCORE_MAX;
			}

			if((n >= 20) && (n <= 30)) {
				if((capture[4] + s) <= SCORE_MAX) capture[4] += s;
				else capture[4] = SCORE_MAX;
			}

			if((n >= 31) && (n <= 46)) {
				if((capture[5] + s) <= SCORE_MAX) capture[5] += s;
				else capture[5] = SCORE_MAX;
			}
			
			if((n >= 47) && (n <= 66)) {
				if((capture[6] + s) <= SCORE_MAX) capture[6] += s;
				else capture[6] = SCORE_MAX;
			}
			
			if((n >= 67) && (n <= 100)) {
				if((capture[7] + s) <= SCORE_MAX) capture[7] += s;		//  
				else capture[7] = SCORE_MAX;
			}

			
		}
		
			capture[0] += capture[0]/5;
			capture[7] -= capture[7]/3;
		
			//==================================================+
			//   ""				|
			//==================================================+
			if (sensitivity != 1) {
				maxover = 0;
				for(uint8_t y = 0; y < 8; y++) {
					over = 0;
					if(capture[y] > THRESHOLD) capture[y] -= THRESHOLD;		//  ,      
					else capture[y] = 0;
					capture[y] = capture[y] / ( 200 / sensitivity );		//   ,    ( DIVIDER / sensitivity )       (   ,  )
					if (capture[y] > 63 ) over = capture[y] - 63;			//         
					if (over > maxover) maxover = over;
				}
			}
		
			//==================================================+
			//      		|
			//==================================================+			
			for(uint8_t y = 0; y < 8; y++) {
		
				if (sensitivity != 1 ) {
					if (capture[y] >= maxover) capture[y] -= maxover;				//  ,           
					else capture[y] = 0;
					//capture[y] = (capture[y] / 200) *sensitivity - (32*sensitivity-1);
				}
				else {
					if(capture[y] > THRESHOLD) capture[y] -= THRESHOLD;	//  ,      
					else capture[y] = 0;
					capture[y] = capture[y] / 200;					//  
				}
				
				if(capture[y] > 63) capture[y] = 63;					//   ,  capture[]   63

				if(counter[y]) counter[y]--;							//    
						
				if(capture[y] >= peak[y]) {								//       
					peak[y] = capture[y];
					counter[y] = 2;										//     
				} 				

				if (pause != 2) {
					//cli();
					pwm_setting[y] = pgm_read_word(&pwmtable_10[peak[y]]);	//   pwm_setting[]     
					//sei();
				}

			}
						
			//==================================================+
			//   						|
			//==================================================+
			if((capture[1] > 0) || (capture[2] > 0) || (capture[3] > 0) || (capture[4] > 0) || (capture[5] > 0) || (capture[6] > 0))	{ //   ,    .		
				
				if (pause != 0) fade = 0;
				fadespeed = fadeinitial * faderate;
				
				no_audio = 0;  
				pause = 0;
				pwm_pause = 0;
				
			}
			else{ //   ,         .
				no_audio = 1;
			}
			


	}
}
